<?php
require_once(dirname(__FILE__) . '/php.inc.php');

class mtgPHPGenerator extends mtgGenerator
{
  function makeTargets(mtgMetaInfo $meta)
  {
    $targets = array();

    $codegen = mtg_conf("codegen", null);
    if(!$codegen)
      $codegen = new mtgPHPCodegen();
    $codegen->setMetaInfo($meta);

    $refl = new ReflectionClass($codegen);
    $SHARED_DEPS = array(
      dirname(__FILE__) . '/php.inc.php', 
      dirname(__FILE__) . '/php_tpl.inc.php',
      __FILE__,
      $refl->getFileName()
    );

    mtg_mkdir(mtg_conf('out-dir'));

    $units = array();
    $rpcs = array();
    $files = array();
    foreach($meta->getUnits() as $unit)
    {
      if($unit->object instanceof mtgMetaRPC)
      {
        $rpcs[] = $unit->object;
        $files = array_merge($files, mtg_get_file_deps($meta, $unit));
      }
      else if($unit->object instanceof mtgMetaEnum || $unit->object instanceof mtgMetaStruct)
      {
        $files = array_merge($files, mtg_get_file_deps($meta, $unit));
        $units[] = $unit;
      }
    }
    $files = array_unique($files);

    $bundle = mtg_conf("bundle", null);
    if($bundle && ($units || $rpcs))
      $targets[] = mtg_new_bundle($bundle, array_merge($SHARED_DEPS, $files), array('gen_php_bundle', $codegen, $units, $rpcs, mtg_conf("inc-dir"))); 

    return $targets;
  }
}

function gen_php_struct($OUT, array $DEPS, mtgCodegen $codegen, mtgMetaInfoUnit $unit)
{
  return $codegen->genUnit($unit);
}

function gen_php_bundle($OUT, array $DEPS, mtgPHPCodegen $codegen, array $units, array $rpcs, $inc_dir)
{
  $bundle = '';
  $class_map = '';
  $packet_map = '';
  $units_src = '';

  $map = array();
  foreach($units as $unit)
  {
    $class_id = $unit->object->getClassId();
    $class_name = $unit->object->getName();
    if(isset($map[$class_id]))
      throw new Exception("Duplicating class id '$class_id'($class_name vs {$map[$class_id]})");
    $map[$class_id] = $class_name;

    $class_map .= "case " . 1*$class_id . ": return \"$class_name\";\n";
    
    if($unit->object instanceof mtgMetaEnum || $unit->object instanceof mtgMetaStruct)
      $units_src .= $codegen->genUnit($unit);
  }

  foreach($rpcs as $rpc)
  {
    $code = $rpc->getCode();
    $name = $rpc->getReq()->getName();

    $packet_map .= "case " . 1*$code . ": return new $name(\$data);\n";
      
    $units_src .= $codegen->genRPC($rpc);
  }

  $templater = new mtg_php_templater();
  $tpl = $templater->tpl_packet_bundle();
  return mtg_fill_template($tpl, array('%bundle%' => $bundle, 
                                       '%units_src%' => $units_src,
                                       '%class_map%' => $class_map,
                                       '%packet_map%' => $packet_map));
}

